import numpy as np
import function
import random

"""
RNN化バージョン0509作成s-PFCのための層内結合ありのネットワーク
rnn_neuronの複製0519
"""

# ニューロン単独
# このプログラムでは行列で定義されたニューロンの出力結果を返す
# ユニットの定義は上層で行う
# 引数として上から結合加重Wとバイアスbを受け取る
# forによる繰り返しは行わないあくまで一回のみ計算

"""
基本パラメーター
**************************
"""
Tau = 10
dt = 0.01
random_del = 0.5


class neuron:
    def __init__(self):
        self.random_del = 1
        self.dt = 0.01
        self.Tau = 10
        self.V = None
        self.S = None
        # 基本関数の定義と各パラメーターの設定
        self.function = function.function()
        self.function.def_parameter(
            self.Tau, self.dt, self.random_del)

    def def_parameter(self, Tau, dt, random_del):
        self.random_del = random_del
        self.dt = dt
        self.Tau = Tau
        self.function.def_parameter(
            self.Tau, self.dt, self.random_del)
        # print("**neuron")
        # print(self.random_del)

    def forward(self, v, si, W, b, mode="None"):
        noise = self.random_del * (random.random()-0.5)
        # 変更点:層内結合の追加
        RI = -noise + np.dot(si, W) + b
        v = self.function.eular_V(v, RI)
        s = self.function.sigmoid(v)
        if mode == "RI":
            return v, s, RI
        return v, s

    def forward_dry(self, v, si, W, b, mode="None"):
        noise = self.random_del * (random.random()-0.5)
        #print("si", si)
        # 変更点:層内結合の追加
        RI = -noise + np.dot(si, W) + b
        #print("RI:", RI)
        v += RI
        s = self.function.sigmoid(v)
        if mode == "RI":
            return v, s, RI
        return v, s

    def forward_it(self, v, s, W, b, input, mode="None"):
        noise = self.random_del * (random.random()-0.5)
        RI = -noise + np.dot(s, W) + input
        v = self.function.eular_V(v, RI)
        s = self.function.sigmoid(v)
        if mode == "RI":
            return v, s, RI
        return v, s



# coding: utf-8
import numpy as np


def sigmoid(x):
    return 1 / (1 + np.exp(-x-1e-7))


def sigmoid_grad(z):
    return sigmoid(z) * (1 - sigmoid(z))


def sigmoid_back(z, dz):
    return dz*sigmoid(z) * (1 - sigmoid(z))


def tanh(x):
    y = (np.exp(x)-np.exp(-x))/(np.exp(x)+np.exp(-x))
    return y


def tanh_grad(x):
    y = x*(1-x**2)
    return y

# 勾配クリッピング


def clip_grads(grads, max_norm=0.25):
    done = 0
    total_norm = 0
    for i in range(len(grads)):
        total_norm += np.sum(grads[i] ** 2)
    total_norm = np.sqrt(total_norm)

    rate = max_norm / (total_norm + 1e-6)
    if rate < 1:
        done = 1
        for i in range(len(grads)):
            grads[i] *= (rate+1)

    return grads, done



import numpy as np
import test_rnn_realtime as net
import time

# 近似データの定義
high_value = 1.0
low_value = 0.0
category_num = 2
steps_num = 3


def make_input_4data(high_value, low_value, category_num):
    one_cate_data = np.empty((0, 0, category_num))
    high_element = np.array([[high_value]])
    low_element = np.array([[low_value]])
    much = np.array([1, 0])
    non_much = np.array([0, 1])

    input_data = np.empty((0, 4, category_num))
    target_data = np.empty((0, 2))
    for q in range(category_num):
        for i in range(category_num):
            for j in range(category_num):
                for k in range(category_num):
                    one_cate_data = np.empty((0, category_num))
                    one_low_data = np.empty((1, 0))
                    for l in range(category_num):
                        if l == q:
                            one_low_data = np.hstack(
                                (one_low_data, high_element))
                        else:
                            one_low_data = np.hstack(
                                (one_low_data, low_element))
                    one_cate_data = np.vstack((one_cate_data, one_low_data))
                    one_low_data = np.empty((1, 0))
                    for l in range(category_num):
                        if l == i:
                            one_low_data = np.hstack(
                                (one_low_data, high_element))
                        else:
                            one_low_data = np.hstack(
                                (one_low_data, low_element))
                    one_cate_data = np.vstack((one_cate_data, one_low_data))
                    one_low_data = np.empty((1, 0))
                    for l in range(category_num):
                        if l == j:
                            one_low_data = np.hstack(
                                (one_low_data, high_element))
                        else:
                            one_low_data = np.hstack(
                                (one_low_data, low_element))
                    one_cate_data = np.vstack((one_cate_data, one_low_data))
                    one_low_data = np.empty((1, 0))
                    for l in range(category_num):
                        if l == k:
                            one_low_data = np.hstack(
                                (one_low_data, high_element))
                        else:
                            one_low_data = np.hstack(
                                (one_low_data, low_element))
                    one_cate_data = np.vstack((one_cate_data, one_low_data))

                    one_cate_data_3 = np.array([one_cate_data])
                    input_data = np.vstack((input_data, one_cate_data_3))
                    if i == j and i == k and q == j:
                        addtarget = much
                    else:
                        addtarget = non_much
                    target_data = np.vstack((target_data, addtarget))

    return input_data, target_data


def make_input_3data(high_value, low_value, category_num, steps_num):
    one_cate_data = np.empty((0, 0, category_num))
    high_element = np.array([[high_value]])
    low_element = np.array([[low_value]])
    much = np.array([1, 0])
    non_much = np.array([0, 1])

    input_data = np.empty((0, steps_num, category_num))
    target_data = np.empty((0, 2))
    for q in range(category_num):
        for i in range(category_num):
            for j in range(category_num):
                one_cate_data = np.empty((0, category_num))
                one_low_data = np.empty((1, 0))
                for l in range(category_num):
                    if l == q:
                        one_low_data = np.hstack(
                            (one_low_data, high_element))
                    else:
                        one_low_data = np.hstack(
                            (one_low_data, low_element))
                one_cate_data = np.vstack((one_cate_data, one_low_data))
                one_low_data = np.empty((1, 0))
                for l in range(category_num):
                    if l == i:
                        one_low_data = np.hstack(
                            (one_low_data, high_element))
                    else:
                        one_low_data = np.hstack(
                            (one_low_data, low_element))
                one_cate_data = np.vstack((one_cate_data, one_low_data))
                one_low_data = np.empty((1, 0))
                for l in range(category_num):
                    if l == j:
                        one_low_data = np.hstack(
                            (one_low_data, high_element))
                    else:
                        one_low_data = np.hstack(
                            (one_low_data, low_element))
                one_cate_data = np.vstack((one_cate_data, one_low_data))
                one_low_data = np.empty((1, 0))

                one_cate_data_3 = np.array([one_cate_data])
                input_data = np.vstack((input_data, one_cate_data_3))
                if i == j and q == j:
                    addtarget = much
                else:
                    addtarget = non_much
                target_data = np.vstack((target_data, addtarget))

    return input_data, target_data


def make_input_2data(high_value, low_value, category_num, steps_num):
    one_cate_data = np.empty((0, 0, category_num))
    high_element = np.array([[high_value]])
    low_element = np.array([[low_value]])
    much = np.array([1, 0])
    non_much = np.array([0, 1])

    input_data = np.empty((0, steps_num, category_num))
    target_data = np.empty((0, 2))
    for q in range(category_num):
        for i in range(category_num):

            one_cate_data = np.empty((0, category_num))
            one_low_data = np.empty((1, 0))
            for l in range(category_num):
                if l == q:
                    one_low_data = np.hstack(
                        (one_low_data, high_element))
                else:
                    one_low_data = np.hstack(
                        (one_low_data, low_element))
            one_cate_data = np.vstack((one_cate_data, one_low_data))
            one_low_data = np.empty((1, 0))
            for l in range(category_num):
                if l == i:
                    one_low_data = np.hstack(
                        (one_low_data, high_element))
                else:
                    one_low_data = np.hstack(
                        (one_low_data, low_element))
            one_cate_data = np.vstack((one_cate_data, one_low_data))
            one_low_data = np.empty((1, 0))

            one_cate_data_3 = np.array([one_cate_data])
            input_data = np.vstack((input_data, one_cate_data_3))
            if i == q:
                addtarget = much
            else:
                addtarget = non_much
            target_data = np.vstack((target_data, addtarget))

    return input_data, target_data


def add_traning_data(input_data, target_data, category_num, chunk):
    much = np.array([1, 0])
    non_much = np.array([0, 1])
    for r in range(category_num ** (chunk - 1) - 2):
        for i in range(category_num):
            for j in range(1, category_num):
                non_much_num = ((i + j) % category_num) * category_num ** 3 + \
                    i * category_num ** 2 + i * category_num + i
                print(input_data[non_much_num])
                input_data = np.vstack(
                    (input_data, np.array([input_data[non_much_num]])))
                target_data = np.vstack(
                    (target_data, np.array([target_data[non_much_num]])))

                test_num = i * category_num ** 3 + i * category_num ** 2 + i * category_num + i
                print(input_data[test_num])
                for ex in range(1):
                    input_data = np.vstack(
                        (input_data, np.array([input_data[test_num]])))
                    target_data = np.vstack(
                        (target_data, np.array([target_data[test_num]])))
    return input_data, target_data


def iadd_traning_data(input_data, target_data, category_num, steps_num):
    much = np.array([1, 0])
    non_much = np.array([0, 1])
    iinput_data = np.empty((0, steps_num, category_num))
    itarget_data = np.empty((0, 2))
    for r in range(1):
        for i in range(category_num):
            for j in range(1, category_num):
                non_much_num = ((i + j) % category_num) * category_num ** 3 + \
                    i * category_num ** 2 + i * category_num + i
                print("non_much_num:", non_much_num)
                print(input_data[non_much_num])
                for ex in range(1):
                    iinput_data = np.vstack(
                        (iinput_data, np.array([input_data[non_much_num]])))
                    itarget_data = np.vstack(
                        (itarget_data, np.array([target_data[non_much_num]])))

                test_num = i * category_num ** 3 + i * category_num ** 2 + i * category_num + i
                print(input_data[test_num])
                for ex in range(1):
                    iinput_data = np.vstack(
                        (iinput_data, np.array([input_data[test_num]])))
                    itarget_data = np.vstack(
                        (itarget_data, np.array([target_data[test_num]])))
    return iinput_data, itarget_data


def iadd_traning_3data(input_data, target_data, category_num, steps_num):
    much = np.array([1, 0])
    non_much = np.array([0, 1])
    iinput_data = np.empty((0, steps_num, category_num))
    itarget_data = np.empty((0, 2))
    for r in range(1):
        for i in range(category_num):
            for j in range(1, category_num):
                non_much_num = ((i + j) % category_num) * category_num ** 2 + \
                    i * category_num + i
                print("non_much_num:", non_much_num)
                print(input_data[non_much_num])
                iinput_data = np.vstack(
                    (iinput_data, np.array([input_data[non_much_num]])))
                itarget_data = np.vstack(
                    (itarget_data, np.array([target_data[non_much_num]])))

                test_num = i * category_num ** 2 + i * category_num + i
                print(input_data[test_num])
                for ex in range(1):
                    iinput_data = np.vstack(
                        (iinput_data, np.array([input_data[test_num]])))
                    itarget_data = np.vstack(
                        (itarget_data, np.array([target_data[test_num]])))
    return iinput_data, itarget_data


def iadd_traning_2data(input_data, target_data, category_num, steps_num):
    much = np.array([1, 0])
    non_much = np.array([0, 1])
    iinput_data = np.empty((0, steps_num, category_num))
    itarget_data = np.empty((0, 2))
    for r in range(1):
        for i in range(category_num):
            for j in range(1, category_num):
                non_much_num = ((i + j) % category_num) * category_num ** 1 + i
                print("non_much_num:", non_much_num)
                print(input_data[non_much_num])
                iinput_data = np.vstack(
                    (iinput_data, np.array([input_data[non_much_num]])))
                itarget_data = np.vstack(
                    (itarget_data, np.array([target_data[non_much_num]])))

                test_num = i * category_num ** 1 + i
                print(input_data[test_num])
                for ex in range(1):
                    iinput_data = np.vstack(
                        (iinput_data, np.array([input_data[test_num]])))
                    itarget_data = np.vstack(
                        (itarget_data, np.array([target_data[test_num]])))
    return iinput_data, itarget_data


def task_3back(high_value, low_value, category_num):
    one_cate_data = np.empty((0, 0, category_num))
    high_element = np.array([[high_value]])
    low_element = np.array([[low_value]])
    t = np.zeros((category_num, category_num))
    for i in range(category_num):
        t[i, i] = 1

    input_data = np.empty((0, 3, category_num))
    target_data = np.empty((0, category_num))
    for q in range(category_num):
        for i in range(category_num):
            for j in range(category_num):
                one_cate_data = np.empty((0, category_num))
                one_low_data = np.empty((1, 0))
                for l in range(category_num):
                    if l == q:
                        one_low_data = np.hstack(
                            (one_low_data, high_element))
                    else:
                        one_low_data = np.hstack(
                            (one_low_data, low_element))
                one_cate_data = np.vstack((one_cate_data, one_low_data))
                one_low_data = np.empty((1, 0))
                for l in range(category_num):
                    if l == i:
                        one_low_data = np.hstack(
                            (one_low_data, high_element))
                    else:
                        one_low_data = np.hstack(
                            (one_low_data, low_element))
                one_cate_data = np.vstack((one_cate_data, one_low_data))
                one_low_data = np.empty((1, 0))
                for l in range(category_num):
                    if l == j:
                        one_low_data = np.hstack(
                            (one_low_data, high_element))
                    else:
                        one_low_data = np.hstack(
                            (one_low_data, low_element))
                one_cate_data = np.vstack((one_cate_data, one_low_data))
                one_low_data = np.empty((1, 0))

                one_cate_data_3 = np.array([one_cate_data])
                input_data = np.vstack((input_data, one_cate_data_3))
                addtarget = t[q]
                target_data = np.vstack((target_data, addtarget))

    return input_data, target_data


def f3_23back(high_value, low_value, category_num, steps_num):
    one_cate_data = np.empty((0, 0, category_num))
    high_element = np.array([[high_value]])
    low_element = np.array([[low_value]])
    t = np.zeros((category_num, category_num))
    for i in range(category_num):
        t[i, i] = 1

    input_data = np.empty((0, steps_num, category_num))
    target_data = np.empty((0, category_num*2))
    for q in range(category_num):
        for i in range(category_num):
            for j in range(category_num):
                one_cate_data = np.empty((0, category_num))
                one_low_data = np.empty((1, 0))
                for l in range(category_num):
                    if l == q:
                        one_low_data = np.hstack(
                            (one_low_data, high_element))
                    else:
                        one_low_data = np.hstack(
                            (one_low_data, low_element))
                one_cate_data = np.vstack((one_cate_data, one_low_data))
                one_low_data = np.empty((1, 0))
                for l in range(category_num):
                    if l == i:
                        one_low_data = np.hstack(
                            (one_low_data, high_element))
                    else:
                        one_low_data = np.hstack(
                            (one_low_data, low_element))
                one_cate_data = np.vstack((one_cate_data, one_low_data))
                one_low_data = np.empty((1, 0))
                for l in range(category_num):
                    if l == j:
                        one_low_data = np.hstack(
                            (one_low_data, high_element))
                    else:
                        one_low_data = np.hstack(
                            (one_low_data, low_element))
                one_cate_data = np.vstack((one_cate_data, one_low_data))
                one_low_data = np.empty((1, 0))

                one_cate_data_3 = np.array([one_cate_data])
                input_data = np.vstack((input_data, one_cate_data_3))
                addtarget = np.hstack((t[q], t[i]))
                target_data = np.vstack((target_data, addtarget))

    return input_data, target_data



# coding: utf-8
import numpy as np


def optimizer_SGD(lr, params, grads):
    for i in range(len(params)):
        params[i] -= lr * grads[i]

    return params


class SGD:
    def __init__(self, lr=0.01):
        self.lr = lr

    def update(self, params, grads):
        for i in range(len(params)):
            params[i] -= self.lr * grads[i]

        return params

    def viewlr(self):
        return self.lr

    def change_lr(self, lr):
        self.lr = lr


class AdaGrad:

    """AdaGrad"""

    def __init__(self, lr=0.01):
        self.lr = lr
        self.h = None
        self.lrs = None

    def update(self, params, grads):
        if self.h is None:
            self.h = {}
            self.lrs = {}
            for i in range(len(params)):
                self.h[i] = np.zeros_like(grads[i])
                self.lrs[i] = np.zeros_like(grads[i])

        for i in range(len(params)):

            self.h[i] += grads[i] * grads[i]
            self.lrs[i] = self.lr / (np.sqrt(self.h[i] + 1e-7))
            params[i] -= self.lrs[i] * grads[i]
        # print(self.h)

        return params

    def viewlr(self):
        return self.lrs

    def change_lr(self, lr):
        self.lr = lr


class NormGrad:

    def __init__(self, lr=0.01):
        self.lr = lr
        self.h = None
        self.lrs = None
        self.cont = 0
        self.grads = None

    def update(self, params, grads):
        if self.h is None:
            self.h = {}
            self.lrs = {}
            for i in range(len(params)):
                self.h[i] = abs(grads[i])
                self.lrs[i] = np.zeros_like(grads[i])

        if 0 < self.cont < 100:
            for i in range(len(params)):
                self.h[i] += abs(grads[i])
                self.lrs[i] = self.lr

        if self.cont == 100:
            for i in range(len(params)):
                self.h[i] = self.h[i]/100
                self.lrs[i] = self.lr

        if self.cont > 100:
            for i in range(len(params)):
                self.lrs[i] = self.lr * abs(grads[i]) / self.h[i]

        for i in range(len(params)):
            params[i] -= self.lrs[i] * grads[i]
        # print(self.h)

        self.grads = grads

        self.cont = self.cont + 1

        return params

    def viewlr(self):
        return self.lrs

    def change_lr(self, lr):
        self.lr = lr



from optimizer import optimizer_SGD, AdaGrad, NormGrad, SGD
import numpy as np
from functions import sigmoid, sigmoid_back, clip_grads


class Loss:
    def __init__(self):
        self.Loss = None
        self.dout = None

    def forward(self, out, t):
        self.Loss = 1/2 * np.sum((out - t)**2)
        self.dout = out - t
        return self.Loss

    def backward(self):
        return self.dout


class RNNneuron:
    def __init__(self, W, Wh, b):
        # 引数として受けた重みとバイアスをself.aramsに格納
        self.params = [W, Wh, b]
        # 更新前に勾配をまとめてオプティマイザーに送るための入れ物（中身はparamsに対応している必要あり）
        self.grads = [np.zeros_like(W), np.zeros_like(Wh), np.zeros_like(b)]
        # クラス外へ中身を持っていくための入れ物
        self.F_container = np.empty(0)
        self.B_container = np.empty(0)
        # RNN層の中身の入れ物
        self.dh_prev = None
        # 学習率の格納
        self.lr = 0.01
        # オプティマイザーの定義（初期値SGD）
        self.optimizer = SGD(self.lr)
        # クリッピングの実行フラグ
        self.clipper = 0
        # 勾配クリッピングのしきい値（初期値0.02)
        self.NormGrad = 0.02

    def forward(self, x, h_prev):
        # クラスの初期化時に格納した重みとバイアスの取り出し
        W, Wh, b = self.params
        # yはニューロン内部の値
        #f = open("E:\研究一時ファイル\BP\TEST_1120\Fh.txt", mode="a")
        if h_prev is None:
            y = np.dot(x, W) + b
        else:
            y = np.dot(h_prev, Wh) + np.dot(x, W) + b

        #w = "\nWh:" + str(Wh) + "\nh_prev:" + str(h_prev) + "\n:" + str(y)

        # f.write(w)

        # Zが出力
        z = sigmoid(y)
        self.h_prev = z
        self.F_container = [W, Wh, b, x, y, z]
        return z, self.F_container

    def backward(self, dz, h_prev):
        #f = open("E:\研究一時ファイル\BP\TEST_1120\Wh.txt", mode="a")
        W, Wh, b, x, y, z = self.F_container
        dh_prev = self.dh_prev
        # 過去時刻からの勾配の合算
        if dh_prev is None:
            dz = dz
        else:
            dz = dh_prev + dz
        # 出力部の逆伝搬（シグモイド版）
        dy = sigmoid_back(z, dz)
        db = dy
        dW = np.dot(x.T, dy)
        dx = np.dot(dy, W.T)
        dWh = np.dot(h_prev.T, dy)
        dh_prev = np.dot(dy, Wh.T)

        #w = "\ndWh:" + str(dWh) + "\nh_prev:" + str(h_prev) + "\ndy:" + str(dy)

        # f.write(w)

        # 勾配クリッピングの実行
        self.drads, self.clipper = clip_grads(self.grads, self.NormGrad)

        self.dh_prev = dh_prev

        # self.gradsに更新に行かう勾配を格納
        self.grads[0][...] = dW
        self.grads[1][...] = dWh
        self.grads[2][...] = db

        # オプティマイザーによりself.paramsの値を更新
        # self.params = optimizer_SGD(self.lr, self.params, self.grads)
        self.params = self.optimizer.update(self.params, self.grads)
        # すべての結果をself.containerに格納
        self.container = [dy, db, dW, dWh, dx]

        # f.close

        return dx, self.container

    def setlr(self, lr, model=0):
        self.lr = lr
        if model == 0:
            self.optimizer = SGD(self.lr)
        elif model == 1:
            self.optimizer = AdaGrad(self.lr)
        elif model == 2:
            self.optimizer = NormGrad(self.lr)

    def viewlr(self):
        return self.optimizer.viewlr()

    def change_lr(self, New_lr):
        self.optimizer.change_lr(New_lr)

    def reset(self):
        self.h_prev = None
        self.dh_prev = None

    def clipper_Chech(self):
        return self.clipper

    def change_NormGrad(self, NormGrad):
        # 勾配クリッピングのしきい値の変更
        self.NormGrad = NormGrad


class BPneuron:
    def __init__(self, W, b):
        # 引数として受けた重みとバイアスをself.aramsに格納
        self.params = [W, b]
        # 更新前に勾配をまとめてオプティマイザーに送るための入れ物（中身はparamsに対応している必要あり）
        self.grads = [np.zeros_like(W), np.zeros_like(b)]
        # クラス外へ中身を持っていくための入れ物
        self.container = np.empty(0)
        # 学習率の格納
        self.lr = 0.01
        self.optimizer = AdaGrad(self.lr)

    def forward(self, x):
        # クラスの初期化時に格納した重みとバイアスの取り出し
        W, b = self.params
        # yはニューロン内部の値
        y = np.dot(x, W)+b
        # Zが出力
        z = sigmoid(y)
        self.container = [W, b, x, y, z]
        return z, self.container

    def backward(self, dz):
        W, b, x, y, z = self.container
        # 出力部の逆伝搬（シグモイド版）
        dy = sigmoid_back(z, dz)
        db = dy
        dW = np.dot(x.T, dy)
        dx = np.dot(dy, W.T)

        # self.gradsに更新に行かう勾配を格納
        self.grads[0][...] = dW
        self.grads[1][...] = db

        # オプティマイザーによりself.paramsの値を更新
        # self.params = optimizer_SGD(self.lr, self.params, self.grads)
        self.params = self.optimizer.update(self.params, self.grads)
        # すべての結果をself.containerに格納
        self.container = [dy, db, dW, dx]

        return dx, self.container

    def setlr(self, lr, model=0):
        self.lr = lr
        if model == 0:
            self.optimizer = SGD(self.lr)
        elif model == 1:
            self.optimizer = AdaGrad(self.lr)
        elif model == 2:
            self.optimizer = NormGrad(self.lr)

    def viewlr(self):
        return self.optimizer.viewlr()

    def change_lr(self, New_lr):
        self.optimizer.change_lr(New_lr)



from rnnclass import BPneuron, Loss, RNNneuron
import matplotlib.pyplot as plt
import numpy as np
from random import random
import matplotlib.pyplot as plt
from IPython.display import clear_output


class test_network:
    def __init__(self, H, O, W12_size):

        # 重みとバイアスの定義
        W12 = W12_size*np.random.rand(H, O)-W12_size/2
        b2 = np.zeros(O)

        # モデルの生成
        self.Third_layer = BPneuron(W12, b2)
        self.test_loss_layer = Loss()

        self.loss_memo = []
        self.b_lr_memo = []
        self.dx_memo = np.empty(0)
        self.bo_memo = np.empty(0)
        self.dbo_memo = np.empty(0)
        self.W12_memo = np.empty(0)

        self.lr = 0.01

        self.Who = None
        self.b = None

        # Abeloss用のlossの入れもの
        self.loss_container = 0

        # 学習率の減数率指数
        self.lr_index = 2

        # 誤差の最小範囲
        self.min_loss = 0.003

    def traning(self, test_data, target_data, epoch):
        # 学習に使う配列の決定
        # print("test")
        for i in range(epoch):
            test_number = np.random.randint(target_data.shape[0])
            ix = test_data[test_number]
            t = np.array([target_data[test_number]])
            out2, container2 = self.Third_layer.forward(ix)
            W, b, x, y, z = container2
            loss = self.test_loss_layer.forward(out2, t)
            self.loss_container += loss
            dout = self.test_loss_layer.backward()
            if i % 100 == 0 and i != 0:
                if (self.loss_container / 100/0.25) > self.min_loss:
                    New_lr = self.lr * \
                        (self.loss_container / 100/0.25) ** self.lr_index

            if i % 500 == 0 and i != 0:
                print("loss:", dout, "epoch:", i)
                print("_________")
                print("tnum", test_number)
                print("out2:", out2)
                print("t:", t)
                print("x:", ix)
                # print("lrs1:", lrs1)
                # print("lrs2:", lrs2)
                print("New_lr:", New_lr)
                print("_________")
                # self.b_lr_memo.append(lrs1[0][0][0])

            if i % 5000 == 0:
                clear_output()

            self.loss_memo.append(loss)
            dx, containe = self.Third_layer.backward(dout)

        self.Who = W
        self.b = b

        return self.loss_memo

    def output_w(self):
        return self.Who, self.b

    def setlr(self, lr, model=0):
        """
        model=0:SGD
        model=1:AdaGrad
        model=2:NormGrad
        """
        self.lr = lr
        self.Third_layer.setlr(lr, model)

    def dx(self):
        return self.dx_memo

    def bo(self):
        return self.bo_memo

    def W_12(self):
        return self.W12_memo

    def set_swquwnce_length(self, sq):
        self.sequence_length = sq

    def set_Wandb(self, W12, b2):
        self.Third_layer = BPneuron(W12, b2)
        self.test_loss_layer = Loss()



import math
import numpy as np
import random

# 基本関数

"""
****************************************
 基本パラメーター
 必ず上層でdef\parameterで定義すること
"""

# ここまで


class function:
    def __int__(self):
        self.Tau = 10
        self.dt = 0.01
        self.random_del = 1

    def def_parameter(self, Tau, dt, random_del):
        self.Tau = Tau
        self.dt = dt
        self.random_del = random_del
        # print("**func")
        # print(self.random_del)

    def sigmoid(self, x):
        y = 1/(1 + np.exp(-5*(x-0.5)))
        return y

    def f_V(self, x, RI):
        sum = 0
        sum = -x + RI
        return (sum / self.Tau)
        print(RI)

    def eular_V(self, x, i):
        x += self.dt * self.f_V(x, i)
        return x

    def noise(self):
        noise = self.random_del * (random.random()-0.5)
        return noise



import numpy as np
import rnn_neuron as nu
import function

# f_FPC_unit: a one unit of f-PFC,2

W = np.array([[0, 1], [0.917, 0]])


class rnn_layer_unit:
    def __init__(self):
        self.random_del = 1
        self.dt = 0.01
        self.Tau = 10
        self.W = None
        self.Wr = None
        self.b = None
        self.V = None
        self.S = None
        self.one_layer = nu.neuron()
        self.one_layer.def_parameter(
            self.Tau, self.dt, self.random_del)
        self.function = function.function()
        self.function.def_parameter(
            self.Tau, self.dt, self.random_del)
        self.mode = None

    def def_parameter(self, Tau, dt, random_del):
        self.random_del = random_del
        self.dt = dt
        self.Tau = Tau
        self.one_layer.def_parameter(
            self.Tau, self.dt, self.random_del)
        # print("**f=pfc")
        # print(self.random_del)

    def initiate(self, v, b, w, wr, mode=None):
        self.W = w
        self.Wr = wr
        self.V = v
        self.S = self.function.sigmoid(v)
        self.b = b
        self.mode = mode

    def forward(self, si):
        self.V, self.S = self.one_layer.forward(
            self.V, si, self.S, self.W, self.Wr, self.b, self.mode)
        return self.V, self.S



from rnnclass import BPneuron, Loss, RNNneuron
import numpy as np
from IPython.display import clear_output
from rnn_layer_unit import rnn_layer_unit
from non_layer_unit import non_layer_unit


class li_rnn_networks:
    def __init__(self, I, H, O, W01_size, W11_size, W12_size, sequence_length):
        # 重みとバイアスの定義
        self.W01 = W01_size*np.random.rand(I, H)-W01_size/2
        self.W11 = W11_size*np.random.rand(H, H)-W11_size/2
        self.W12 = W12_size*np.random.rand(H, O)-W12_size/2
        self.V2 = np.zeros((1, H))
        self.V3 = np.zeros((1, O))
        self.b1 = np.zeros(H)
        self.b2 = np.zeros(O)

        # モデルの生成
        self.Seccond_layer = RNNneuron(self.W01, self.W11, self.b1)
        self.Third_layer = BPneuron(self.W12, self.b2)
        self.test_loss_layer = Loss()
        self.I = I
        self.H = H
        self.O = O

        self.lSeccond_layer = rnn_layer_unit()
        self.lThird_layer = non_layer_unit()

        self.loss_memo = []
        self.loss_memo2 = []
        self.b_lr_memo = []
        self.dx_memo = np.empty(0)
        self.bo_memo = np.empty(0)
        self.dbo_memo = np.empty(0)
        self.W12_memo = np.empty(0)

        self.lr = 0.01

        # RNNの繰り返し期間sequence_lengthの格納
        self.sequence_length = sequence_length
        # RNNの時刻ごとの出力の入れ物
        self.h_prev = np.empty((sequence_length, H))

        # Abeloss用のlossの入れもの
        self.loss_container = 0

        # 学習率の減数率指数
        self.lr_index = 2

        # 誤差の最小範囲
        self.min_loss = 0.003

    def def_parameter(self, Tau_H, Tau_O, dt, random_del, Step_length):
        self.random_del = random_del
        self.dt = dt
        self.Step_length = Step_length
        self.Tau_H = Tau_H
        self.Tau_O = Tau_O
        self.lSeccond_layer.def_parameter(Tau_H, self.dt, self.random_del)
        self.lThird_layer.def_parameter(Tau_O, self.dt, self.random_del)

        self.lSeccond_layer.initiate(self.V2, self.b1, self.W01, self.W11)
        self.lThird_layer.initiate(self.V3, self.b2, self.W12)

    def set_Wandb(self, W01, W11, W12, b1, b2):
        self.W01 = W01
        self.W11 = W11
        self.W12 = W12
        self.b2 = b1
        self.b3 = b2
        self.Seccond_layer = RNNneuron(self.W01, self.W11, self.b1)
        self.Third_layer = BPneuron(self.W12, self.b2)

    def output_W(self):
        return self.W01, self.W11, self.W12, self.b1, self.b2

    def network_reset(self):
        self.V2 = np.zeros((1, self.H))
        self.V3 = np.zeros((1, self.O))
        self.lSeccond_layer.initiate(self.V2, self.b1, self.W01, self.W11)
        self.lThird_layer.initiate(self.V3, self.b2, self.W12)

    def traning(self, test_data, target_data, epoch):
        self.test_data = test_data
        self.target_data = target_data
        # 学習に使う配列の決定
        # print("test")
        for i in range(epoch):
            test_number = np.random.randint(target_data.shape[0])
            ix = test_data[test_number]
            t = np.array([target_data[test_number]])
            out1 = None
            for step in range(self.sequence_length):
                input_data = ix[step].reshape([1, self.I])
                out1, container1 = self.Seccond_layer.forward(input_data, out1)
                # print(out1.shape)
                self.h_prev[step] = out1
            out2, container2 = self.Third_layer.forward(out1)

            loss = self.test_loss_layer.forward(out2, t)
            self.loss_container += loss
            dout = self.test_loss_layer.backward()
            if i % 100 == 0 and i != 0:
                if (self.loss_container / 100/0.25) > self.min_loss:
                    New_lr = self.lr * \
                        (self.loss_container / 100/0.25) ** self.lr_index

                else:
                    New_lr = 0.0001

                self.loss_container = 0
                self.Seccond_layer.change_lr(New_lr)
                self.Third_layer.change_lr(New_lr)

            if i % 500 == 0 and i != 0:
                print("loss:", dout, "epoch:", i)
                print("_________")
                print("out1:", out1)
                print("out2:", out2)
                print("t:", t)
                print("x:", ix)
                # print("lrs1:", lrs1)
                # print("lrs2:", lrs2)
                print("New_lr:", New_lr)
                print("_________")
                # self.b_lr_memo.append(lrs1[0][0][0])
                clipping = self.Seccond_layer.clipper_Chech()
                if clipping == 1:
                    print("Clipping was done")

            if i % 5000 == 0:
                clear_output()

            self.loss_memo.append(loss)
            dx, containe = self.Third_layer.backward(dout)
            for step in range(self.sequence_length):
                h_prev = np.array(
                    [self.h_prev[self.sequence_length - step - 1]])
                dinput, containe = self.Seccond_layer.backward(dx, h_prev)
            # self.dx_memo = np.append(self.dx_memo, dx)
            # self.bo_memo = np.append(self.bo_memo, b)
            # self.W12_memo = np.append(self.W12_memo, W)
            self.Seccond_layer.reset()
        # plt.plot(self.b_lr_memo)

        self.W01, self.W11, self.b1, x, y, z = container1
        self.W12, self.b2, x, y, z = container2

        return self.loss_memo

    def to_li_para(self, epoch, interruption=True):
        self.loss_memo2 = []
        self.gene_li_out()
        # 学習に使う配列の決定
        # print("test")
        for i in range(epoch):
            test_number = np.random.randint(self.itarget_data.shape[0])
            ix = self.itest_data[test_number]
            t = np.array([self.itarget_data[test_number]])
            out2, container2 = self.Third_layer.forward(ix)
            W, b, x, y, z = container2
            loss = self.test_loss_layer.forward(out2, t)
            if (self.loss_container < loss):
                self.loss_container = loss
            dout = self.test_loss_layer.backward()
            if i % 100 == 0 and i != 0:
                if (interruption is True):
                    if ((self.loss_container) < 0.2):
                        print(self.loss_container)
                        return self.loss_memo2
                if (self.loss_container / 0.25) > self.min_loss:
                    New_lr = self.lr * \
                        (self.loss_container / 0.25) ** self.lr_index

            if i % 500 == 0 and i != 0:
                print("loss:", dout, "epoch:", i)
                print("_________")
                print("tnum", test_number)
                print("out2:", out2)
                print("t:", t)
                print("x:", ix)
                print("New_lr:", New_lr)
                print("_________")

            if i % 5000 == 0:
                clear_output()

            self.loss_memo2.append(loss)
            dx, containe = self.Third_layer.backward(dout)

        self.Who = W
        self.b2 = b

        self.Seccond_layer = RNNneuron(self.W01, self.W11, self.b1)
        self.Third_layer = BPneuron(self.W12, self.b2)

        return self.loss_memo2

    def li_forward(self, test_data, target_data, test_num=None):
        memo_out1 = np.empty((0, self.H))
        memo_out2 = np.empty((0, self.O))
        self.network_reset()

        out1 = None
        cont = 0
        dtT = int(1/self.dt)

        if (test_num != None):
            test_data = test_data[test_num]
            print("test_num:", test_num)

        for seaquence in range(self.sequence_length):
            input_data = test_data[seaquence].reshape((1, self.I))
            for t in range(self.Step_length * dtT):
                ix = input_data.reshape((1, self.I))
                v2, out1 = self.lSeccond_layer.forward(ix)
                v3, out2 = self.lThird_layer.forward(out1)
                if t % dtT == 0:
                    memo_out1 = np.vstack((memo_out1, out1))
                    memo_out2 = np.vstack((memo_out2, out2))
                # print("memo")
        # plt.plot(self.b_lr_memo)

        self.out1 = out1
        self.out2 = out2

        return memo_out1, memo_out2

    def setlr(self, lr, model=0):
        """
        model=0:SGD
        model=1:AdaGrad
        model=2:NormGrad
        """
        self.lr = lr
        self.Seccond_layer.setlr(lr, model)
        self.Third_layer.setlr(lr, model)

    def dx(self):
        return self.dx_memo

    def bo(self):
        return self.bo_memo

    def W_12(self):
        return self.W12_memo

    def set_swquwnce_length(self, sq):
        self.sequence_length = sq

    def sequence_prediction(self, x):
        # 学習に使う配列の決定
        out1 = self.h_prev[self.sequence_length - 1]
        input_data = np.array([[x[0][0]]])
        out1, container1 = self.Seccond_layer.forward(input_data, out1)
        self.h_prev[self.sequence_length - 1] = out1
        out2, container2 = self.Third_layer.forward(out1)
        return out2

    def set_NormGrad(self, NormGrad):
        # 勾配クリッピングのしきい値の変更
        self.Seccond_layer.change_NormGrad(NormGrad)

    def set_adjust(self, lr_index, min_loss):
        self.lr_index = lr_index
        self.min_loss = min_loss

    def gene_li_out(self):
        self.itest_data = np.empty((0, 1, self.H))
        self.itarget_data = np.empty((0, self.I))
        for i in range(self.test_data.shape[0]):
            self.def_parameter(self.Tau_H, self.Tau_O, self.dt,
                               self.random_del, self.Step_length)
            self.network_reset()
            memo = self.li_forward(self.test_data, self.target_data, i)
            test = np.round(self.output_H(), decimals=3)
            test = np.array([test])
            self.itest_data = np.vstack((self.itest_data, test))
            addtarget = self.target_data[i]
            print(addtarget)
            self.itarget_data = np.vstack((self.itarget_data, addtarget))

        print(self.itest_data.shape)
        print(self.itarget_data.shape)

    def output_H(self):
        return self.out1

    def output_O(self):
        return self.out2


def task_3back(high_value, low_value, category_num):
    one_cate_data = np.empty((0, 0, category_num))
    high_element = np.array([[high_value]])
    low_element = np.array([[low_value]])
    t = np.zeros((category_num, category_num))
    for i in range(category_num):
        t[i, i] = 1

    input_data = np.empty((0, 3, category_num))
    target_data = np.empty((0, category_num))
    for q in range(category_num):
        for i in range(category_num):
            for j in range(category_num):
                one_cate_data = np.empty((0, category_num))
                one_low_data = np.empty((1, 0))
                for l in range(category_num):
                    if l == q:
                        one_low_data = np.hstack(
                            (one_low_data, high_element))
                    else:
                        one_low_data = np.hstack(
                            (one_low_data, low_element))
                one_cate_data = np.vstack((one_cate_data, one_low_data))
                one_low_data = np.empty((1, 0))
                for l in range(category_num):
                    if l == i:
                        one_low_data = np.hstack(
                            (one_low_data, high_element))
                    else:
                        one_low_data = np.hstack(
                            (one_low_data, low_element))
                one_cate_data = np.vstack((one_cate_data, one_low_data))
                one_low_data = np.empty((1, 0))
                for l in range(category_num):
                    if l == j:
                        one_low_data = np.hstack(
                            (one_low_data, high_element))
                    else:
                        one_low_data = np.hstack(
                            (one_low_data, low_element))
                one_cate_data = np.vstack((one_cate_data, one_low_data))
                one_low_data = np.empty((1, 0))

                one_cate_data_3 = np.array([one_cate_data])
                input_data = np.vstack((input_data, one_cate_data_3))
                addtarget = t[q]
                target_data = np.vstack((target_data, addtarget))

    return input_data, target_data


def f3_23back(high_value, low_value, category_num, steps_num):
    one_cate_data = np.empty((0, 0, category_num))
    high_element = np.array([[high_value]])
    low_element = np.array([[low_value]])
    t = np.zeros((category_num, category_num))
    for i in range(category_num):
        t[i, i] = 1

    input_data = np.empty((0, steps_num, category_num))
    target_data = np.empty((0, category_num*2))
    for q in range(category_num):
        for i in range(category_num):
            for j in range(category_num):
                one_cate_data = np.empty((0, category_num))
                one_low_data = np.empty((1, 0))
                for l in range(category_num):
                    if l == q:
                        one_low_data = np.hstack(
                            (one_low_data, high_element))
                    else:
                        one_low_data = np.hstack(
                            (one_low_data, low_element))
                one_cate_data = np.vstack((one_cate_data, one_low_data))
                one_low_data = np.empty((1, 0))
                for l in range(category_num):
                    if l == i:
                        one_low_data = np.hstack(
                            (one_low_data, high_element))
                    else:
                        one_low_data = np.hstack(
                            (one_low_data, low_element))
                one_cate_data = np.vstack((one_cate_data, one_low_data))
                one_low_data = np.empty((1, 0))
                for l in range(category_num):
                    if l == j:
                        one_low_data = np.hstack(
                            (one_low_data, high_element))
                    else:
                        one_low_data = np.hstack(
                            (one_low_data, low_element))
                one_cate_data = np.vstack((one_cate_data, one_low_data))
                one_low_data = np.empty((1, 0))

                one_cate_data_3 = np.array([one_cate_data])
                input_data = np.vstack((input_data, one_cate_data_3))
                addtarget = np.hstack((t[q], t[i]))
                target_data = np.vstack((target_data, addtarget))

    return input_data, target_data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

import sys
from setuptools import setup, find_packages


setup(
    name='li_rn_networks',
    version='0.1',
    description='neural networks. It have recurrent connections and leaky integrator neuron.',
    long_description_content_type='text/markdown',
    author='picker.',
    author_email='picker@pickerlav.jp',
    url='https://github.com/shibuiwilliam/py-kaomoji.git',
    packages=['li_rn_networks'],
    include_package_data=True,
)



import numpy as np
from random import random
import matplotlib.pyplot as plt
from IPython.display import clear_output
from rnn_layer_unit import rnn_layer_unit
from non_layer_unit import non_layer_unit


class test_network:
    def __init__(self, I, H, O, W01_size, W11_size, W12_size, sequence_length, w_path=None):
        self.O = O
        self.H = H
        self.I = I

        # 重みとバイアスの定義
        self.W01 = W01_size*np.random.rand(I, H)-W01_size/2
        self.W11 = W11_size*np.random.rand(H, H)-W11_size/2
        self.W12 = W12_size*np.random.rand(H, O)-W12_size/2
        self.V2 = np.zeros((1, H))
        self.V3 = np.zeros((1, O))
        self.b2 = np.zeros((1, H))
        self.b3 = np.zeros((1, O))

        # モデルの生成
        self.Seccond_layer = rnn_layer_unit()
        self.Third_layer = non_layer_unit()

        # １層、２層も初期化
        self.Seccond_layer.initiate(self.V2, self.b2, self.W01, self.W11)
        self.Third_layer.initiate(self.V3, self.b3, self.W12)

        self.loss_memo = []
        self.b_lr_memo = []
        self.dx_memo = np.empty(0)
        self.bo_memo = np.empty(0)
        self.dbo_memo = np.empty(0)
        self.W12_memo = np.empty(0)

        self.out1 = None
        self.out2 = None

        # realtime化に必要なパラメーター
        self.random_del = 1
        self.dt = 0.01
        self.Tau = 100
        self.Step_length = 100
        self.Seccond_layer.def_parameter(self.Tau, self.dt, self.random_del)
        self.Seccond_layer.def_parameter(self.Tau, self.dt, self.random_del)

        self.lr = 0.01

        # RNNの繰り返し期間sequence_lengthの格納
        self.sequence_length = sequence_length
        # RNNの時刻ごとの出力の入れ物
        self.h_prev = np.empty((sequence_length, H))

        # Abeloss用のlossの入れもの
        self.loss_container = 0

        # 学習率の減数率指数
        self.lr_index = 2

        # 誤差の最小範囲
        self.min_loss = 0.003

    def def_parameter(self, Tau, dt, random_del, Step_length):
        self.random_del = random_del
        self.dt = dt
        self.Tau = Tau
        self.Step_length = Step_length
        self.Seccond_layer.def_parameter(self.Tau, self.dt, self.random_del)
        self.Seccond_layer.def_parameter(self.Tau, self.dt, self.random_del)

    def set_Wandb(self, W01, W11, W12, bh, bo):
        self.W01 = W01
        self.W11 = W11
        self.W12 = W12
        self.b2 = bh
        self.b3 = bo
        self.Seccond_layer.initiate(self.V2, self.b2, self.W01, self.W11)
        self.Third_layer.initiate(self.V3, self.b3, self.W12)

    def network_reset(self):
        self.V2 = np.zeros((1, self.H))
        self.V3 = np.zeros((1, self.O))
        self.Seccond_layer.initiate(self.V2, self.b2, self.W01, self.W11)
        self.Third_layer.initiate(self.V3, self.b3, self.W12)

    def forward(self, test_data, target_data, clock):
        # 学習に使う配列の決定
        # print("test")
        memo_out = []
        #print("test_number:", test_number)
        #ix = test_data[test_number]

        out1 = None
        cont = 0
        dtT = int(1/self.dt)
        #print("dtT", dtT)
        #print("sequence_length", self.sequence_length)
        # for step in range(self.sequence_length):
        #input = ix[step].reshape([1, self.I])
        # if cont == 0:
        #clock = int(100 / self.dt)
        #print("clock", clock)
        out_memo = np.zeros((0, self.H))

        #est_number = np.random.randint(target_data.shape[0])

        for t in range(clock):
            if(t % 1000 == 0):
                print(test_data[0, t])
            input = test_data[0, t].reshape((1, self.I))
            v2, out1 = self.Seccond_layer.forward(input)
            v3, out2 = self.Third_layer.forward(out1)
            if t % dtT == 0:
                memo_out.append(out1)
                # print("memo")
        # plt.plot(self.b_lr_memo)

        self.out1 = out1
        self.out2 = out2

        return memo_out

    def output_H(self):
        return self.out1

    def output_O(self):
        return self.out2

    def setlr(self, lr, model=0):
        """
        model=0:SGD
        model=1:AdaGrad
        model=2:NormGrad
        """
        self.lr = lr
#       self.Seccond_layer.setlr(lr, model)
#        self.Third_layer.setlr(lr, model)

    def dx(self):
        return self.dx_memo

    def bo(self):
        return self.bo_memo

    def W_12(self):
        return self.W12_memo

    def set_swquwnce_length(self, sq):
        self.sequence_length = sq

    def sequence_prediction(self, x):
        # 学習に使う配列の決定
        out1 = self.h_prev[self.sequence_length - 1]
        input = np.array([[x[0][0]]])
        out1, container1 = self.Seccond_layer.forward(input, out1)
        self.h_prev[self.sequence_length - 1] = out1
        out2, container2 = self.Third_layer.forward(out1)
        return out2

    def set_NormGrad(self, NormGrad):
        # 勾配クリッピングのしきい値の変更
        self.Seccond_layer.change_NormGrad(NormGrad)

    def set_adjust(self, lr_index, min_loss):
        self.lr_index = lr_index
        self.min_loss = min_loss



import numpy as np
import non_neuron as nu
import function
import random

# f_FPC_unit: a one unit of f-PFC,2
# rmm_layer_unitの複製0519

W = np.array([[0, 1], [0.917, 0]])


class non_layer_unit:
    def __init__(self):
        self.random_del = 1
        self.dt = 0.01
        self.Tau = 10
        self.W = None
        self.b = None
        self.V = None
        self.S = None
        self.si = None
        self.one_layer = nu.neuron()
        self.one_layer.def_parameter(
            self.Tau, self.dt, self.random_del)
        self.function = function.function()
        self.function.def_parameter(
            self.Tau, self.dt, self.random_del)
        self.mode = None

        self.Loss = None
        self.dout = None

        self.lr = 0.01

    def set_lr(self, lr):
        self.lr = lr

    def loss(self, t):
        #print("S:", self.S)
        #print("t:", t)
        self.Loss = 1/2 * np.sum((self.S - t)**2)
        #print("Loss", self.Loss)

    def def_parameter(self, Tau, dt, random_del):
        self.random_del = random_del
        self.dt = dt
        self.Tau = Tau
        self.one_layer.def_parameter(
            self.Tau, self.dt, self.random_del)
        # print("**f=pfc")
        # print(self.random_del)

    def initiate(self, v, b, w, mode=None):
        self.W = w
        self.V = v
        self.S = self.function.sigmoid(v)
        self.b = b
        self.mode = mode

    def sgd(self, dW, db):
        #print("W1", self.W[0])
        self.W -= self.lr * dW
        self.b -= self.lr * db
        #print("W2", self.W[0])
        #print("lr", self.lr)

    def forward(self, si):
        self.si = si
        self.V, self.S = self.one_layer.forward(
            self.V, si, self.W, self.b)
        #print("S", self.S)
        return self.V, self.S

    def forward_dry(self, si):
        self.si = si
        self.V, self.S = self.one_layer.forward_dry(
            self.V, si, self.W, self.b)
        #print("S*", self.S)
        return self.V, self.S

    def backward(self, t):
        dy = self.function.sigmoid_back(self.S, self.Loss)
        dW = np.dot(self.si.T, dy)
        dx = np.dot(dy, self.W.T)
        self.sgd(dW, dy)

    def output_loss(self, t):
        self.dout = self.S - t
        return self.dout[0][0]

    def printS(self):
        print(self.S)



import math
import numpy as np
import random
import non_neuron as nu

# f_FPC_unit: a one unit of f-PFC,2

W = np.array([[0, 1], [0.917, 0]])


class f_PFC_unit:
    def __init__(self, V_d, V_c):
        self.random_del = 1
        self.dt = 0.01
        self.Tau = 10
        self.Wl = np.array([[0, 0.99], [0.922, 0]])
        self.Wh = np.array([[0, 1.02], [0.875, 0]])
        self.b = np.array([[0.0, 0.0]])
        self.V_d = V_d
        self.V_c = V_c
        self.f_FPC_unit_d = nu.neuron()
        self.f_FPC_unit_d.def_parameter(
            self.Tau, self.dt, self.random_del)
        self.f_FPC_unit_c = nu.neuron()
        self.f_FPC_unit_c.def_parameter(
            self.Tau, self.dt, self.random_del)

    def def_parameter(self, Tau, dt, random_del):
        self.random_del = random_del
        self.dt = dt
        self.Tau = Tau
        self.f_FPC_unit_c.def_parameter(
            self.Tau, self.dt, self.random_del)
        self.f_FPC_unit_d.def_parameter(
            self.Tau, self.dt, self.random_del)

    def forward(self, S_d, S_c, ITd, ITc):
        b_d = np.zeros_like(S_d)
        b_c = np.zeros_like(S_c)
        self.V_d, S_d = self.f_FPC_unit_d.forward_it(
            self.V_d, S_d, self.Wl, b_d, ITd)
        self.V_c, S_c = self.f_FPC_unit_d.forward_it(
            self.V_c, S_c, self.Wl, b_c, ITc)
        return S_d, S_c


"""
    def forward(self, S_d, S_c, ITd, ITc):
        b_d = np.zeros_like(S_d)
        b_c = np.zeros_like(S_c)
        self.V_d, S_d = self.f_FPC_unit_d.forward(
            self.V_d, S_d, self.Wl, b_d, ITd)
        self.V_c, S_c = self.f_FPC_unit_c.forward(
            self.V_c, S_c, self.Wl, b_c, ITc)
        return S_d, S_c
"""
